@@ -1,6 +1,6 @@
 /* Test harness for pipe-filter-ii.
 
-   Copyright (C) 2009-2013 Free Software Foundation, Inc.
+   Copyright (C) 2009-2020 Free Software Foundation, Inc.
    Written by Paolo Bonzini <bonzini@gnu.org>, 2009.
 
    This program is free software: you can redistribute it and/or modify
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -27,7 +27,6 @@
 #include <signal.h>
 
 #include "full-write.h"
-#include "progname.h"
 #include "macros.h"
 
 struct locals
@@ -87,8 +86,6 @@ main (int argc, char **argv)
 {
   const char *path[] = { NULL, NULL };
 
-  set_program_name (argv[0]);
-
   ASSERT (argc == 2);
 
   /* Test writing to a nonexistent program traps sooner or later.  */
