@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.2.2.  */
+/* A Bison parser, made by GNU Bison 3.6.4.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,11 +31,12 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-/* Undocumented macros, especially those whose name start with YY_,
-   are private implementation details.  Do not rely on them.  */
+/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
+   especially those whose name start with YY_ or yy_.  They are
+   private implementation details that can be changed or removed.  */
 
-#ifndef YY_YY_CLDR_PLURAL_H_INCLUDED
-# define YY_YY_CLDR_PLURAL_H_INCLUDED
+#ifndef YY_YY_CLDR_PLURAL_TAB_H_INCLUDED
+# define YY_YY_CLDR_PLURAL_TAB_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -43,43 +45,35 @@
 extern int yydebug;
 #endif
 
-/* Token type.  */
+/* Token kinds.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
   enum yytokentype
   {
-    AND = 258,
-    OR = 259,
-    RANGE = 260,
-    ELLIPSIS = 261,
-    OTHER = 262,
-    AT_INTEGER = 263,
-    AT_DECIMAL = 264,
-    KEYWORD = 265,
-    INTEGER = 266,
-    DECIMAL = 267,
-    OPERAND = 268
+    YYEMPTY = -2,
+    YYEOF = 0,                     /* "end of file"  */
+    YYerror = 256,                 /* error  */
+    YYUNDEF = 257,                 /* "invalid token"  */
+    AND = 258,                     /* AND  */
+    OR = 259,                      /* OR  */
+    RANGE = 260,                   /* RANGE  */
+    ELLIPSIS = 261,                /* ELLIPSIS  */
+    OTHER = 262,                   /* OTHER  */
+    AT_INTEGER = 263,              /* AT_INTEGER  */
+    AT_DECIMAL = 264,              /* AT_DECIMAL  */
+    KEYWORD = 265,                 /* KEYWORD  */
+    INTEGER = 266,                 /* INTEGER  */
+    DECIMAL = 267,                 /* DECIMAL  */
+    OPERAND = 268                  /* OPERAND  */
   };
+  typedef enum yytokentype yytoken_kind_t;
 #endif
-/* Tokens.  */
-#define AND 258
-#define OR 259
-#define RANGE 260
-#define ELLIPSIS 261
-#define OTHER 262
-#define AT_INTEGER 263
-#define AT_DECIMAL 264
-#define KEYWORD 265
-#define INTEGER 266
-#define DECIMAL 267
-#define OPERAND 268
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-
 union YYSTYPE
 {
-#line 127 "./cldr-plural.y" /* yacc.c:1906  */
+#line 129 "./cldr-plural.y"
 
   char *sval;
   struct cldr_plural_condition_ty *cval;
@@ -90,9 +84,9 @@ union YYSTYPE
   struct cldr_plural_range_list_ty *rval;
   int ival;
 
-#line 94 "cldr-plural.h" /* yacc.c:1906  */
-};
+#line 88 "cldr-plural.tab.h"
 
+};
 typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
@@ -102,4 +96,4 @@ typedef union YYSTYPE YYSTYPE;
 
 int yyparse (struct cldr_plural_parse_args *arg);
 
-#endif /* !YY_YY_CLDR_PLURAL_H_INCLUDED  */
+#endif /* !YY_YY_CLDR_PLURAL_TAB_H_INCLUDED  */
